from PySide2 import QtWidgets, QtCore

from ciomax.components import widgets


class IntFieldGrp(QtWidgets.QWidget):

    def __init__(self, **kwargs):
        super(IntFieldGrp, self).__init__()

        self.hidable = kwargs.get("hidable")
        self.enablable = kwargs.get("enablable")
        if self.hidable:
            self.enablable = False


        layout = QtWidgets.QHBoxLayout()
        self.setLayout(layout)

        layout.addWidget(widgets.FormLabel(kwargs.get("label", "")))

        # hidable checkbox
        if self.hidable or self.enablable:
            self.display_checkbox = QtWidgets.QCheckBox()
            layout.addWidget(self.display_checkbox)
            self.display_checkbox.stateChanged.connect(self.set_active)

        # field
        self.field = QtWidgets.QSpinBox()
        if kwargs.get("minimum") is not None:
            self.field.setMinimum(kwargs.get("minimum"))
        self.field.setSingleStep(kwargs.get("step", 1))
        self.field.setValue(kwargs.get("default", 0))
        layout.addWidget(self.field)

        # optional checkbox
        self.checkbox = widgets.add_checkbox(
            layout,
            kwargs.get("checkbox"),
            kwargs.get("check_label", "")
        )

    def set_active(self, value=None):
        if not (self.hidable or self.enablable):
            return
        if self.hidable:
            self._show() if value else self._hide()

        self._enable() if value else self._disable()


    def _hide(self):
        self.display_checkbox.setCheckState(QtCore.Qt.Unchecked)
        self.field.hide()
        if self.checkbox:
            self.checkbox.hide()

    def _show(self):
        self.display_checkbox.setCheckState(QtCore.Qt.Checked)
        self.field.show()
        if self.checkbox:
            self.checkbox.show()

    def _disable(self):
        self.display_checkbox.setCheckState(QtCore.Qt.Unchecked)
        self.field.setEnabled(False)
        if self.checkbox:
            self.checkbox.setEnabled(False)

    def _enable(self):
        self.display_checkbox.setCheckState(QtCore.Qt.Checked)
        self.field.setEnabled(True)
        if self.checkbox:
            self.checkbox.setEnabled(True)




from PySide2 import QtWidgets, QtCore


class FormLabel(QtWidgets.QLabel):

    def __init__(self, label, tooltip=None):
        super(FormLabel, self).__init__()

        self.setText(label)
        self.setFixedWidth(120)
        self.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignBottom)
        self.setIndent(5)

        if tooltip:
            self.setToolTip(tooltip)


def add_checkbox(layout, do_checkbox=True, label=""):
    if do_checkbox:
        checkbox = QtWidgets.QCheckBox(label)
        checkbox.setFixedWidth(80)
        layout.addWidget(checkbox)
        return checkbox
    else:
        layout.addSpacing(85)




import os
VERSION = u"dev.999"
PLUGIN_DIR = os.path.dirname(__file__)
try:
    with open(os.path.join(PLUGIN_DIR, "VERSION")) as version_file:
        VERSION = version_file.read().strip()
except BaseException:
    pass




from PySide2 import QtWidgets, QtGui
import json


class PreviewTab(QtWidgets.QScrollArea):

    def __init__(self,dialog):
        super(PreviewTab, self).__init__()
    
        widget = QtWidgets.QWidget()
        self.dialog = dialog
        self.setWidget(widget)
        self.setWidgetResizable(1)

        layout = QtWidgets.QHBoxLayout()
        widget.setLayout(layout)

        self.text_area = QtWidgets.QTextEdit()

        self.text_area.setReadOnly(True)
        self.text_area.setWordWrapMode(QtGui.QTextOption.NoWrap)
        layout.addWidget(self.text_area)

        # self.populate(JSON)

    def populate(self, submission):
        self.text_area.setText(json.dumps(submission, indent=3))







from PySide2 import QtWidgets, QtGui, QtCore

from ciomax.sections.collapsible_section import CollapsibleSection
from ciomax.components.text_field_grp import TextFieldGrp
from ciomax.components.combo_box_grp import ComboBoxGrp, InstanceTypeComboBoxGrp
from ciocore import data as coredata

class GeneralSection(CollapsibleSection):
    ORDER = 10

    def __init__(self, dialog):
        super(GeneralSection, self).__init__(dialog, "General", expanded=True)

        self.title_component = TextFieldGrp(
            label="Job Title", tooltip="The title that appears in the Conductor web dashboard on the jobs page.")

        self.project_component = ComboBoxGrp(label="Project")
        self.instance_type_component = InstanceTypeComboBoxGrp()

        self.destination_component = TextFieldGrp(
            label="Override Destination",
            enablable=True,
            directory=True,
            placeholder="Path where renders are saved to"
        )

        self.content_layout.addWidget(self.title_component)
        self.content_layout.addWidget(self.project_component)
        self.content_layout.addWidget(self.instance_type_component)
        self.content_layout.addWidget(self.destination_component)

        self.set_project_model()
        self.set_instance_type_model()
        self.configure_signals()

    def set_project_model(self):
        if not coredata.valid():
            print "Invalid projects data"
            return False

        model = QtGui.QStandardItemModel()
        for project in coredata.data()["projects"] or []:
            model.appendRow(QtGui.QStandardItem(project))
        self.project_component.set_model(model)
        return True

    def set_instance_type_model(self):
        if not coredata.valid():
            print "Invalid Instance Types"
            return False

        model = QtGui.QStandardItemModel()
        cpu_item = QtGui.QStandardItem("CPU")
        gpu_item = QtGui.QStandardItem("GPU")
        for entry in coredata.data()["instance_types"] or []:
            if entry.get("gpu"):
                gpu_item.appendRow((QtGui.QStandardItem(
                    entry["description"]), QtGui.QStandardItem(entry["name"])))
            else:
                cpu_item.appendRow((QtGui.QStandardItem(
                    entry["description"]), QtGui.QStandardItem(entry["name"])))
        model.appendRow(cpu_item)
        model.appendRow(gpu_item)

        self.instance_type_component.set_model(model)

        return True

    def configure_signals(self):
        """Write to store when values change"""
        self.title_component.field.editingFinished.connect(
            self.on_title_change)

        self.project_component.combobox.currentTextChanged.connect(
            self.on_project_change)

        self.instance_type_component.combobox_machine.currentTextChanged.connect(
            self.on_instance_type_change)

        self.instance_type_component.checkbox.stateChanged.connect(
            self.on_preemptible_change)

        self.destination_component.display_checkbox.stateChanged.connect(
            self.on_override_destination_change)

        self.destination_component.field.editingFinished.connect(
            self.on_destination_change)

        self.destination_component.button.clicked.connect(
            self.on_destination_change)

    def on_title_change(self):
        self.dialog.store.set_title(self.title_component.field.text())

    def on_project_change(self, value):
        self.dialog.store.set_project(value)

    def on_instance_type_change(self, value):
        short_name = self.instance_type_component.get_item_name(value)
        self.dialog.store.set_instance_type(short_name)

    def on_preemptible_change(self, value):
        self.dialog.store.set_preemptible(value > 0)

    def on_override_destination_change(self, value):
        self.dialog.store.set_override_destination(value > 0)

    def on_destination_change(self):
        self.dialog.store.set_destination(
            self.destination_component.field.text())

    def populate_from_store(self):
        store = self.dialog.store
        self.title_component.field.setText(store.title())

        self.destination_component.field.setText(store.destination())

        self.destination_component.set_active(store.override_destination())

        self.project_component.set_by_text(store.project())
        self.instance_type_component.set_by_text(
            store.instance_type(), column=1)

        self.instance_type_component.checkbox.setCheckState(
            QtCore.Qt.Checked if store.preemptible() else QtCore.Qt.Unchecked)

    def resolve(self, expander):

        return {
            "job_title": expander.evaluate(self.title_component.field.text()),
            "output_path": self.destination_component.field.text(),
            "project": self.project_component.combobox.currentText(),
            "instance_type":  self.instance_type_component.get_current_item_name(),
            "preemptible": self.instance_type_component.checkbox.isChecked()
        }



import re
from PySide2 import QtWidgets, QtGui
import MaxPlus

from ciocore.sequence import Sequence
from ciomax.sections.collapsible_section import CollapsibleSection
from ciomax.components.text_field_grp import TextFieldGrp
from ciomax.components.int_field_grp import IntFieldGrp
SCOUT_AUTO_REGEX = re.compile(r"^auto[, :]+(\d+)$")
from ciomax.store import DEFAULT_TASK_TEMPLATE 
from ciocore.expander import Expander

class FramesSection(CollapsibleSection):
    ORDER = 30

    def __init__(self, dialog):
        super(FramesSection, self).__init__(dialog, "Frames", expanded=False)

        self.main_sequence = None
        self.scout_sequence = None

        self.chunk_size_component = IntFieldGrp(
            label="Chunk size", default=1, minimum=1)
        self.custom_range_component = TextFieldGrp(
            label="Use custom range", hidable=True)
        self.scout_frames_component = TextFieldGrp(
            label="Use scout frames", enablable=True)

        self.content_layout.addWidget(self.chunk_size_component)
        self.content_layout.addWidget(self.custom_range_component)
        self.content_layout.addWidget(self.scout_frames_component)

        self.configure_signals()

    def configure_signals(self):
        """Write to store when values change"""
        self.chunk_size_component.field.valueChanged.connect(
            self.on_chunk_size_change)

        self.custom_range_component.field.editingFinished.connect(
            self.on_custom_range_change)

        self.custom_range_component.display_checkbox.stateChanged.connect(
            self.on_use_custom_range_change)

        self.scout_frames_component.field.editingFinished.connect(
            self.on_scout_frames_change)

        self.scout_frames_component.display_checkbox.stateChanged.connect(
            self.on_use_scout_frames_change)

    def on_chunk_size_change(self, value):
        self.dialog.store.set_chunk_size(value)
        self._set_sequences()

    def on_custom_range_change(self):
        self.dialog.store.set_custom_range(
            self.custom_range_component.field.text())
        self._set_sequences()

    def on_use_custom_range_change(self, value):
        self.dialog.store.set_use_custom_range(value > 0)
        self._set_sequences()

    def on_scout_frames_change(self):
        self.dialog.store.set_scout_frames(
            self.scout_frames_component.field.text())
        self._set_sequences()

    def on_use_scout_frames_change(self, value):
        self.dialog.store.set_use_scout_frames(value > 0)
        self._set_sequences()

    def _set_sequences(self):
        """Convert all the frames parameters into two Sequence objects.

        self.main_sequence
        self.scout_sequence
        """
        chunk_size = self.chunk_size_component.field.value()
        if self.custom_range_component.display_checkbox.isChecked():
            frame_spec = self.custom_range_component.field.text().strip()
        else:
            frame_spec = scene_frame_spec()
        try:
            self.main_sequence = Sequence.create(
                frame_spec, chunk_size=chunk_size, chunk_strategy="progressions")
        except (ValueError, TypeError):
            self.main_sequence = None

        if self.scout_frames_component.display_checkbox.isChecked():
            scout_spec = self.scout_frames_component.field.text().strip()
            match = SCOUT_AUTO_REGEX.match(scout_spec)
            if match:
                samples = int(match.group(1))

                self.scout_sequence = self.main_sequence.subsample(samples)
            else:
                try:
                    self.scout_sequence = Sequence.create(scout_spec)
                except (ValueError, TypeError):
                    self.scout_sequence = None
        else:
            self.scout_sequence = None

        if self.dialog.main_tab:
            self.dialog.main_tab.section("InfoSection").update(
                self.main_sequence, self.scout_sequence)

    def populate_from_store(self):

        store = self.dialog.store
        self.chunk_size_component.field.setValue(store.chunk_size())

        self.custom_range_component.set_active(store.use_custom_range())
        self.custom_range_component.field.setText(store.custom_range())

        self.scout_frames_component.set_active(store.use_scout_frames())
        self.scout_frames_component.field.setText(store.scout_frames())

        self._set_sequences()


 
    def resolve(self, expander, **kwargs):
        advanced_section = self.dialog.main_tab.section("AdvancedSection")
        template_component = advanced_section.task_template_component
        self._set_sequences()

 
        if  advanced_section.upload_only_checkbox.isChecked():
            return {}
        tasks = []
        max_tasks = kwargs.get("max_tasks", -1)
        
        if template_component.display_checkbox.isChecked():
            template = template_component.field.text()
        else:
            template = DEFAULT_TASK_TEMPLATE

        truncate = max_tasks > -1

        chunks = self.main_sequence.chunks()
        
        context = expander.context

        for i, chunk in enumerate(chunks):
            if truncate and i >= max_tasks:
                break
            task_context = {
                "start": str(chunk.start),
                "end": str(chunk.end),
                "step": str(chunk.step),
                "chunk_length":  str(len(chunk))
            }
            task_context.update(context)
            this_expander = Expander(safe=True, **task_context)

            tasks.append({
                "command": this_expander.evaluate(template),
                "frames": str(chunk)
            })

        return {
            "tasks_data": tasks,
            "scout_frames": ",".join([str(s) for s in self.scout_sequence or []])
            }
            

 

def scene_frame_spec():
    """Get the 3ds_max_frame_range as a spec."""
    ticks = MaxPlus.Animation.GetTicksPerFrame()
    frame_range = MaxPlus.Animation.GetAnimRange()
    return "{:d}-{:d}".format(
        int(frame_range.Start() / ticks),
        int(frame_range.End() / ticks)
    )





from PySide2 import QtWidgets


class FooterButtonsGrp(QtWidgets.QWidget):
 
    def __init__(self, **kwargs):
        super(FooterButtonsGrp, self).__init__()

        layout = QtWidgets.QHBoxLayout()
        self.setLayout(layout)

        self.close_button = QtWidgets.QPushButton("Close")
        self.validate_button = QtWidgets.QPushButton("Validate")
        self.submit_button = QtWidgets.QPushButton("Submit")
        
        layout.addWidget(self.close_button)
        layout.addWidget(self.validate_button)
        layout.addWidget(self.submit_button)



from PySide2 import QtWidgets, QtCore

from ciomax.components import widgets


class SoftwareComboBoxGrp(QtWidgets.QWidget):
    """Combo box triple for host software, renderers, and versions.

    The model is a tree of depth 3, and therefore the second combo box is
    maintained to always contain the children of the model item selected in the
    first combo box and the 3rd combo box is maintained to always contain the
    children of the model item selected in the second combo box.
    """

    def __init__(self):
        """Create the widgets"""
        super(SoftwareComboBoxGrp, self).__init__()

        self.model = None

        layout = QtWidgets.QVBoxLayout()
        self.setLayout(layout)
        top_layout = QtWidgets.QHBoxLayout()
        bottom_layout = QtWidgets.QHBoxLayout()
        layout.addLayout(top_layout)
        layout.addLayout(bottom_layout)

        top_layout.addWidget(widgets.FormLabel("3ds Max version"))
        self.combobox_host = QtWidgets.QComboBox()
        top_layout.addWidget(self.combobox_host)
        top_layout.addSpacing(85)

        bottom_layout.addWidget(widgets.FormLabel("Renderer version"))
        self.combobox_renderer_name = QtWidgets.QComboBox()
        self.combobox_renderer_version = QtWidgets.QComboBox()
        bottom_layout.addWidget(self.combobox_renderer_name)
        bottom_layout.addWidget(self.combobox_renderer_version)
        bottom_layout.addSpacing(85)

        self.combobox_host.currentIndexChanged.connect(self._host_changed)
        self.combobox_renderer_name.currentIndexChanged.connect(self._renderer_changed)

    def set_model(self, model):
        """Attach the model and set the widgets to the appropriate model depth."""
        self.model = model
        self.combobox_host.setModel(self.model)
        self.combobox_renderer_name.setModel(self.model)
        self.combobox_renderer_version.setModel(self.model)

        renderer_model_index = self.model.index(0, 0, QtCore.QModelIndex())
        self.combobox_renderer_name.setRootModelIndex(renderer_model_index)

        version_model_index = self.model.index(0, 0, renderer_model_index)
        self.combobox_renderer_version.setRootModelIndex(version_model_index)

    def set_host_by_text(self, text):
        """Set the host combo box to given 3dsmax version if it is available."""
        if not self.model:
            return
        for row in range(self.model.rowCount()):
            if self.model.item(row).text() == text:
                self.combobox_host.setCurrentIndex(row)
                return
        self.combobox_host.setCurrentIndex(0)

    def set_renderer_by_text(self, text):
        """Set the renderer/veresion combo boxes to given renderer if it is available."""
        if not self.model:
            return
        found_renderer = False
        renderer_text, version_text = text.split(" ")

        host_item = self.model.item(self.combobox_host.currentIndex())
        renderer_row = 0
        for renderer_row in range(host_item.rowCount()):
            if host_item.child(renderer_row, 0).text() == renderer_text:

                found_renderer = True
                self.combobox_renderer_name.setCurrentIndex(renderer_row)
                break
        if not found_renderer:
            self.combobox_renderer_name.setCurrentIndex(0)  # triggers version model change
            self.combobox_renderer_version.setCurrentIndex(0)
            return

        renderer_item = host_item.child(renderer_row, 0)
        for version_row in range(renderer_item.rowCount()):
            if renderer_item.child(version_row, 0).text() == version_text:

                self.combobox_renderer_version.setCurrentIndex(version_row)
                return
        self.combobox_renderer_version.setCurrentIndex(0)

    def get_renderer(self):
        return "{} {}".format(
            self.combobox_renderer_name.currentText(),
            self.combobox_renderer_version.currentText(),
            
        )
        
    ############ PRIVATE ############
    def _host_changed(self, host_row):
        # When host changes, update the selection of renderers 
        if not self.model:
            return

        # remember the text for currently selected renderer
        renderer_text = self.combobox_renderer_name.currentText()

        renderer_model_index = self.model.index(host_row, 0)
        self.combobox_renderer_name.setRootModelIndex(renderer_model_index)

        renderer_row = self._find_renderer_row(host_row, renderer_text)
        # When we set the renderer combo box, _renderer_changed() is called.
        self.combobox_renderer_name.setCurrentIndex(renderer_row)

    def _renderer_changed(self, renderer_row):
        # When renderer changes, update the selection of versions 
        if not self.model:
            return
        host_row = self.combobox_host.currentIndex()

        # remember the text for currently selected version
        version_text = self.combobox_renderer_version.currentText()

        renderer_model_index = self.model.index(host_row, 0)
        version_model_index = self.model.index(
            renderer_row, 0, renderer_model_index)
        self.combobox_renderer_version.setRootModelIndex(version_model_index)

        version_row = self._find_version_row(
            host_row, renderer_row, version_text)
        self.combobox_renderer_version.setCurrentIndex(version_row)

    def _find_renderer_row(self, host_row,  renderer_text):
        # renderer row from text
        host_item = self.model.item(host_row)
        for renderer_row in range(host_item.rowCount()):
            if host_item.child(renderer_row, 0).text() == renderer_text:
                return renderer_row
        return 0

    def _find_version_row(self, host_row, renderer_row, version_text):
        # version row from text
        renderer_item = self.model.item(host_row).child(renderer_row, 0)
        for version_row in range(renderer_item.rowCount()):
            if renderer_item.child(version_row, 0).text() == version_text:
                return version_row
        return 0



 
import os
import MaxPlus
from PySide2 import QtWidgets, QtCore, QtGui
from ciocore.expander import Expander

from ciomax.sections.collapsible_section import CollapsibleSection

from ciomax.sections.general_section import GeneralSection
from ciomax.sections.software_section import SoftwareSection
from ciomax.sections.frames_section import FramesSection
from ciomax.sections.info_section import InfoSection
from ciomax.sections.environment_section import EnvironmentSection
from ciomax.sections.metadata_section import MetadataSection
from ciomax.sections.extra_assets_section import ExtraAssetsSection
from ciomax.sections.advanced_section import AdvancedSection
from ciocore import data as coredata
# from ciomax.store import ConductorStore

from ciocore.gpath import Path

FIXTURES_DIR = os.path.expanduser(os.path.join("~", "Conductor", "fixtures"))


# class MainTab(QtWidgets.QScrollArea):
#     def __init__(self, dialog):
#         super(MainTab, self).__init__()


class MainTab(QtWidgets.QScrollArea):
    """
    Build the tab that contains the main configuration sections.
    """

    def __init__(self, dialog):
        super(MainTab, self).__init__()

        self.dialog = dialog
        coredata.init(product="maya-io")
        coredata.set_fixtures_dir(FIXTURES_DIR)
        coredata.data()

        widget = QtWidgets.QWidget()
        self.setWidget(widget)
        self.setWidgetResizable(1)

        sections_layout = QtWidgets.QVBoxLayout()
        widget.setLayout(sections_layout)

        self._section_classes = sorted(
            CollapsibleSection.__subclasses__(), key=lambda x: x.ORDER)
        self.sections = [cls(self.dialog) for cls in self._section_classes]

        for section in self.sections:
            sections_layout.addWidget(section)

        sections_layout.addStretch()

    def populate_from_store(self):
        for section in self.sections:
            section.populate_from_store()

    def section(self, classname):
        """
        Convenience to find sections by name.

        Makes it easier to allow sections to talk to each other.
        Example: Calculate info from stuff in the frames section
            self.section("InfoSection").calculate(self.section("FramesSection"))

        """

        return next(s for s in self.sections if s.__class__.__name__ == classname)

    def resolve(self):
        submission = {}
        kwargs = {}
        context = self.get_context()
        expander = Expander(safe=True, **context)
        for section in self.sections:
            submission.update(section.resolve(expander, **kwargs))
        return submission

    def get_context(self):
        scenefile = MaxPlus.FileManager.GetFileNameAndPath()
        if scenefile:
            scenefile = Path(scenefile).posix_path()
            scenedir = os.path.dirname(scenefile)
            scenenamex, ext = os.path.splitext(os.path.basename(scenefile))
        else:
            scenefile = "/NOT SAVED"
            scenedir = "/NOT SAVED"
            scenenamex, ext = ["NOT SAVED", ""]
        scenename = "{}{}".format(scenenamex,ext)

        project = MaxPlus.Core.EvalMAXScript(
            'pathConfig.getCurrentProjectFolder()').Get()
        if project:
            project = Path(project).posix_path()
        else:
            project = "/NOT SET"

        result = {
            "scenefile": scenefile,
            "scenedir": scenedir,
            "scenenamex": scenenamex,
            "ext": ext,
            "scenename":scenename,
            "project": project,
            "renderer": MaxPlus.RenderSettings.GetProduction().GetClassName()
        }
        return result




from PySide2 import QtWidgets, QtCore, QtGui

EXPANDED_STYLESHEET = "CollapsibleSection { border: 1px solid #555; border-radius: 2px;}"
COLLAPSED_STYLESHEET = "CollapsibleSection { border: none;}"



class CollapsibleSection(QtWidgets.QFrame):

    def __init__(self, dialog, title, expanded=False):

        super(CollapsibleSection, self).__init__()
        
        self.dialog = dialog
        self.content_area = QtWidgets.QWidget()
        self.content_layout = QtWidgets.QVBoxLayout()
        self.content_area.setLayout(self.content_layout)


        self.toggle_button = QtWidgets.QToolButton()
        self.toggle_button.setIconSize(QtCore.QSize(7,7))

        font=QtGui.QFont()
        font.setBold(True)
        self.toggle_button.setFont(font)

        self._configure_toggle_button(title, expanded)

        self._configure_content_area()
        self._configure_main_layout()

        self.toggle_button.clicked.connect(self.set_expanded)
        self.set_expanded(expanded)

    def build(self):
        raise NotImplementedError

    def set_expanded(self, expand):
        if expand:
            self.toggle_button.setArrowType(QtCore.Qt.DownArrow)
            self.content_area.show()
            self.setStyleSheet(EXPANDED_STYLESHEET)
        else:
            self.toggle_button.setArrowType(QtCore.Qt.RightArrow)
            self.content_area.hide()
            self.setStyleSheet(COLLAPSED_STYLESHEET)

    # PRIVATE
    def _configure_toggle_button(self, title, expanded):
        self.toggle_button.setStyleSheet("QToolButton { border: none; }")
        self.toggle_button.setToolButtonStyle(
            QtCore.Qt.ToolButtonTextBesideIcon)
        self.toggle_button.setText(title)
        self.toggle_button.setCheckable(True)
        self.toggle_button.setChecked(expanded)

    def _configure_content_area(self):
        self.content_area.setSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
            
    def _configure_main_layout(self):
        layout = QtWidgets.QGridLayout()
        layout.setVerticalSpacing(0)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(
            self.toggle_button, 0, 0, 1, 1, QtCore.Qt.AlignLeft)
        layout.addWidget(self.content_area, 1, 0, 1, 3)
        self.setLayout(layout)

    def populate_from_store(self):
        pass
    
    def resolve(self, _):
        return {}


from PySide2 import QtWidgets, QtGui, QtCore

from ciomax.sections.collapsible_section import CollapsibleSection
from ciomax.components.key_value_grp import KeyValueGrpList


class EnvironmentSection(CollapsibleSection):
    ORDER = 60

    def __init__(self, dialog):
        super(EnvironmentSection, self).__init__(dialog, "Extra Environment")

        self.component = KeyValueGrpList(
            checkbox_label="Excl", key_label="Name")
        self.content_layout.addWidget(self.component)
        self.configure_signals()

    def configure_signals(self):
        """Write to store when values change"""
        self.component.edited.connect(self.on_edited)

    def on_edited(self):
        self.dialog.store.set_extra_environment(self.component.entries())

    def get_entries(self, expander):

        return [{
            "name": x[0],
            "value": expander.evaluate(x[1]),
            "merge_policy": "exclusive" if x[2] else "append"
        } for x in self.component.entries()]

    def populate_from_store(self):
        store = self.dialog.store
        self.component.set_entries(store.extra_environment())




from PySide2 import QtWidgets, QtCore

from ciomax.components import widgets


class TextGrp(QtWidgets.QWidget):

    def __init__(self, **kwargs):
        super(TextGrp, self).__init__()

        layout = QtWidgets.QHBoxLayout()
        self.setLayout(layout)

        layout.addWidget(widgets.FormLabel(kwargs.get("label", "")))

        self.text = QtWidgets.QLabel()

        layout.addWidget(self.text)




from ciomax import store
from ciomax.sections import collapsible_section
from ciomax.components import widgets

from ciomax.components import footer_buttons_grp
from ciomax.components import text_field_grp
from ciomax.components import int_field_grp
from ciomax.components import text_grp
from ciomax.components import key_value_grp
from ciomax.components import combo_box_grp
from ciomax.components import software_combo_box_grp
from ciomax.components import checkbox_grp
from ciomax.sections import general_section
from ciomax.sections import software_section
from ciomax.sections import frames_section
from ciomax.sections import info_section
from ciomax.sections import advanced_section
from ciomax.sections import environment_section
from ciomax.sections import metadata_section
from ciomax.sections import extra_assets_section
from ciomax import main_tab
from ciomax import preview_tab

# The order of there reloads is important. 
reload(store)
reload(collapsible_section)
reload(widgets)
reload(footer_buttons_grp)
reload(text_field_grp)
reload(int_field_grp)
reload(text_grp)
reload(key_value_grp)
reload(combo_box_grp)
reload(software_combo_box_grp)
reload(checkbox_grp)
reload(general_section)
reload(software_section)
reload(frames_section)
reload(info_section)
reload(advanced_section)
reload(environment_section)
reload(metadata_section)
reload(extra_assets_section)
reload(main_tab)
reload(preview_tab)



from PySide2 import QtWidgets, QtCore

from ciomax.components import widgets

LETTER_WIDTH = 7
LETTER_PAD = 35


class ModelBaseGrp(QtWidgets.QWidget):
    """
    Base class for  widget groups that use a model.
    """

    def __init__(self, **kwargs):
        super(ModelBaseGrp, self).__init__()

        self.model = None

        layout = QtWidgets.QHBoxLayout()
        self.setLayout(layout)

        layout.addWidget(widgets.FormLabel(kwargs.get("label", "")))

        self.checkbox = widgets.add_checkbox(
            layout,
            kwargs.get("checkbox"),
            kwargs.get("check_label", "")
        )


class ComboBoxGrp(ModelBaseGrp):
    """Single combo box containing model rows."""

    def __init__(self,  **kwargs):
        super(ComboBoxGrp, self).__init__(**kwargs)

        self.combobox = QtWidgets.QComboBox()
        # insrt after label, and before checkbox/spacer
        self.layout().insertWidget(1, self.combobox)

    def set_by_text(self, text, column=0, default=0):
        """set to the row where column contains the given text."""
        for row in range(self.model.rowCount()):
            if self.model.item(row, column).text() == text:
                self.combobox.setCurrentIndex(row)
                return
        self.combobox.setCurrentIndex(default)

    def set_model(self, model):
        self.model = model
        self.combobox.setModel(self.model)


class InstanceTypeComboBoxGrp(ModelBaseGrp):
    """Combo box pair.

    The model is a tree of depth 2, and therefore the second combo box is
    maintained to always contain the children of the model item selected in the
    first combo box.
    """

    def __init__(self):
        kwargs = {
            "label": "Instance Type",
            "checkbox": True,
            "check_label": "Preemptible"
        }

        super(InstanceTypeComboBoxGrp, self).__init__(**kwargs)
        # category = GPU/CPU
        self.combobox_category = QtWidgets.QComboBox()
        self.combobox_machine = QtWidgets.QComboBox()

        # insert widgets before the checkbox.
        self.layout().insertWidget(1, self.combobox_category)
        self.layout().insertWidget(2, self.combobox_machine)

        self.combobox_category.currentIndexChanged.connect(
            self.category_changed)

    def set_model(self, model):
        """Set the model, """
        self.model = model
        self.combobox_category.setModel(self.model)
        self.combobox_machine.setModel(self.model)

        root_width = self.calc_root_width()
        self.combobox_category.setFixedWidth(root_width)

        self.combobox_machine.setRootModelIndex(self.model.index(0, 0))

        self.combobox_category.setCurrentIndex(0)
        self.combobox_machine.setCurrentIndex(0)

    def category_changed(self, index):
        """Change category and set machine to the first in tyhe list."""
        if not self.model:
            return
        self.set_by_indices(index, 0)

    def set_by_indices(self, i, j):

        if not ((self.combobox_category.count() > i) and (self.combobox_machine.count() > j)):
            return
        self.combobox_category.setCurrentIndex(i)
        model_index = self.model.index(i, 0)
        self.combobox_machine.setRootModelIndex(model_index)
        self.combobox_machine.setCurrentIndex(j)

    def set_by_text(self, text, column=0, default=(0, 0)):
        if not self.model:
            return
        for root_row in range(self.model.rowCount()):
            root_item = self.model.item(root_row)
            for row in range(root_item.rowCount()):
                if root_item.child(row, column).text() == text:
                    self.set_by_indices(root_row, row)
                    return

        self.set_by_indices(default[0], default[1])

    def calc_root_width(self):
        letters = 0
        for row in range(self.model.rowCount()):
            curr_letters = len(self.model.item(row).text())
            if curr_letters > letters:
                letters = curr_letters
        return (letters * LETTER_WIDTH) + LETTER_PAD

    def get_item_name(self, long_name):
        for root_row in range(self.model.rowCount()):
            root_item = self.model.item(root_row)
            for row in range(root_item.rowCount()):
                if root_item.child(row, 0).text() == long_name:
                    return root_item.child(row, 1).text()

    def get_current_item_name(self):
        return  self.get_item_name(self.combobox_machine.currentText())




from PySide2 import QtWidgets, QtCore

from ciomax.components import widgets


class CheckboxGrp(QtWidgets.QWidget):

    def __init__(self, **kwargs):
        super(CheckboxGrp, self).__init__()
        layout = QtWidgets.QHBoxLayout()
        self.setLayout(layout)
        num_checkboxes = kwargs.get("checkboxes", 1)
        sublabels = kwargs.get("sublabels", [])

        if len(sublabels) != num_checkboxes:
            sublabels = [""] * num_checkboxes

        layout.addWidget(widgets.FormLabel(kwargs.get("label", "")))

        self.checkboxes = []

        for i in range(num_checkboxes):
            cb = QtWidgets.QCheckBox(sublabels[i])
            self.checkboxes.append(cb)
            layout.addWidget(cb)

        layout.addSpacing(85)




import sys
import os

from PySide2 import QtWidgets, QtCore, QtGui
import qtmax
from pymxs import runtime as rt

# DO NOT SORT IMPORTS
sys.path.append(os.path.dirname(os.path.dirname(__file__)))


from ciomax import reloader
reload(reloader)
# While in development, we reload classes when this file is executed.

from ciomax.components.footer_buttons_grp import FooterButtonsGrp
from ciomax.preview_tab import PreviewTab
from ciomax.main_tab import MainTab
from ciomax.store import ConductorStore


BACKGROUND_COLOR = "rgb(48, 48, 48)"
STYLESHEET = """
QLineEdit {{ background: {bg}; }}
QSpinBox {{ background: {bg}; }}
QListWidget {{ background: {bg}; }}
QToolButton {{ border: none; }}
QTextEdit {{ background: {bg}; }}""".format(bg=BACKGROUND_COLOR)


class ConductorDialog(QtWidgets.QDialog):
    """
    Build the dialog as a child of the Max window.

    We build a tab layout, and the first tab contains the main controls.
    """

    def __init__(self):
        QtWidgets.QDialog.__init__(
            self, QtWidgets.QWidget.find(rt.windows.getMAXHWND()))
        self.setStyleSheet(STYLESHEET)
        self.store = ConductorStore()
        self.setWindowTitle("Conductor")
        self.layout = QtWidgets.QVBoxLayout()
        self.tab_widget = QtWidgets.QTabWidget()

        self.button_row = FooterButtonsGrp()

        self.setLayout(self.layout)
        self.layout.addWidget(self.tab_widget)
        self.layout.addWidget(self.button_row)

        self.main_tab = MainTab(self)
        self.preview_tab = PreviewTab(self)

        self.tab_widget.addTab(self.main_tab, "Configure")
        self.tab_widget.addTab(self.preview_tab, "Preview")

        self.main_tab.populate_from_store()
        self.configure_signals()

    def configure_signals(self):
        self.tab_widget.currentChanged.connect(self.on_tab_change)

        self.button_row.close_button.clicked.connect(self.on_close)
        self.button_row.close_button.clicked.connect(self.on_validate)
        self.button_row.close_button.clicked.connect(self.on_submit)

    def on_tab_change(self, index):
        if index == 1:

            submission = self.main_tab.resolve()
            self.preview_tab.populate(submission)

    def on_close(self):
        print "on_close"

    def on_validate(self):
        print "on_validate"

    def on_submit(self):
        print "on_submit"


def main():

    dlg = ConductorDialog()
    dlg.resize(600, 800)

    # exec_() causes the window to be modal. This means we don't have to manage
    # any communication between max and the dialog like changes to the frame
    # range, while the dialog is open.
    dlg.exec_()


if __name__ == '__main__':
    main()





from PySide2 import QtWidgets, QtGui
from ciocore.sequence import Sequence
from ciomax.sections.collapsible_section import CollapsibleSection
from ciomax.components.text_field_grp import TextFieldGrp


class InfoSection(CollapsibleSection):
    ORDER = 40

    def __init__(self, dialog):
        super(InfoSection, self).__init__(dialog, "Info", expanded=True)

        self.frame_info_component = TextFieldGrp(
            label="Frame info")
        self.scout_info_component = TextFieldGrp(
            label="Scout info")

        self.frame_info_component.field.setEnabled(False)
        self.scout_info_component.field.setEnabled(False)

        self.content_layout.addWidget(self.frame_info_component)
        self.content_layout.addWidget(self.scout_info_component)

    def update(self, main_sequence, scout_sequence):

        if not main_sequence:
            self.frame_info_component.field.setText("INVALID SEQUENCE")
            self.scout_info_component.field.setText("INVALID SEQUENCE")
            return

        frame_count = len(main_sequence)
        task_count = main_sequence.chunk_count()

        frames_info = "spec:{} --- tasks:{:d} --- frames:{:d}".format(
            main_sequence, task_count, frame_count)

        scout_info = "No scout tasks. All tasks will start."

        scout_task_count = 0

        scout_tasks_sequence = None
        if scout_sequence:
            scout_chunks = main_sequence.intersecting_chunks(scout_sequence)
            if scout_chunks:
                scout_tasks_sequence = Sequence.create(
                    ",".join(str(chunk) for chunk in scout_chunks))
                scout_task_count = len(scout_chunks)
                scout_frame_count = len(scout_tasks_sequence)
                scout_info = "spec:{} --- tasks:{:d} --- frames:{:d}".format(
                    scout_tasks_sequence, scout_task_count, scout_frame_count)

        self.frame_info_component.field.setText(frames_info)
        self.scout_info_component.field.setText(scout_info)

    def populate(self, frame_info, scout_info):
        self.frame_info_component.field.setText(frame_info)
        self.scout_info_component.field.setText(scout_info)

  



"""
Encapsulation of a max dummy object to use as a settings store.

The purpose of the store is to make it possible to persist the state of the
dialog (settings) in the scene file.

The Object IDs are abstracted here.
Therefore we can present a public API containing the following methods for each setting:
def attribute():
def set_attribute(value):

The store is NOT the single source of truth while dialog is open. The widgets are.

So the flow is:

- User opens dialog:
- Check if scene has an instance of the conductor_store.
    - If not, make one and reset it to factory defaults.
- Populate the UI from the store.

When the user changes some value, persist it to the store.

"""

import MaxPlus


import os
import json
import time
from ciocore.gpath import Path
from pymxs import runtime as rt
import ciomax.const as k

DEFAULT_TASK_TEMPLATE = u'3dsmax -render "<scenefile>" -renderer "<renderer>" -proj "<project>" -frame <start> <end> <step>'
DEFAULT_AUTOSAVE_TEMPLATE = u"<scenedir>/cio_<scenename>"
DEFAULT_TITLE = u"3dsmax <scenenamex>"

default_script_filename = os.path.expanduser(os.path.join("~", "Conductor", "ciomax", "scripts", "export_ass.py"))

DEFAULT_SCRIPT_COMMAND = u'{} "<project>/<scenenamex>.####.ass" <start> <end>'.format( Path(default_script_filename).posix_path())
STORE_NAME = u"ConductorStore"


# IDS: NOTE Always add to the end - Don't insert! Don't ever reorder or remove
# entries, even if an attribute becomes redundant. If you do it will make old
# scenes incompatible.
X = 2000
TITLE = X = X+1
PROJECT = X = X+1
DESTINATION = X = X+1
EXTRA_ASSETS = X = X+1
INSTANCE_TYPE = X = X+1
PREEMPTIBLE = X = X+1
CHUNK_SIZE = X = X+1
USE_CUSTOM_RANGE = X = X+1
CUSTOM_RANGE = X = X+1
USE_SCOUT_FRAMES = X = X+1
SCOUT_FRAMES = X = X+1
TASK_TEMPLATE = X = X+1
EXTRA_ENVIRONMENT = X = X+1
METADATA = X = X+1
USE_UPLOAD_DAEMON = X = X+1
UPLOAD_ONLY = X = X+1
RETRIES_WHEN_PREEMPTED = X = X+1
USE_AUTOSAVE = X = X+1
AUTOSAVE_FILENAME = X = X+1
AUTOSAVE_CLEANUP = X = X+1
LOCATION_TAG = X = X+1
SHOW_TRACEBACKS = X = X+1
HOST_VERSION = X = X+1
RENDERER_VERSION = X = X+1
EMAILS = X = X+1
USE_EMAILS = X = X+1
USE_FIXTURES = X = X+1
FIXTURES_PATH = X = X+1
OVERRIDE_DESTINATION = X = X+1
OVERRIDE_TASK_TEMPLATE = X = X+1
USE_SCRIPT = X = X+1
SCRIPT_FILENAME = X = X+1


def print_ids():
    print "TITLE ID:", TITLE
    print "PROJECT ID:", PROJECT
    print "DESTINATION ID:", DESTINATION
    print "EXTRA_ASSETS ID:", EXTRA_ASSETS
    print "INSTANCE_TYPE ID:", INSTANCE_TYPE
    print "PREEMPTIBLE ID:", PREEMPTIBLE
    print "CHUNK_SIZE ID:", CHUNK_SIZE
    print "USE_CUSTOM_RANGE ID:", USE_CUSTOM_RANGE
    print "CUSTOM_RANGE ID:", CUSTOM_RANGE
    print "USE_SCOUT_FRAMES ID:", USE_SCOUT_FRAMES
    print "SCOUT_FRAMES ID:", SCOUT_FRAMES
    print "TASK_TEMPLATE ID:", TASK_TEMPLATE
    print "EXTRA_ENVIRONMENT ID:", EXTRA_ENVIRONMENT
    print "METADATA ID:", METADATA
    print "USE_UPLOAD_DAEMON ID:", USE_UPLOAD_DAEMON
    print "UPLOAD_ONLY ID:", UPLOAD_ONLY
    print "RETRIES_WHEN_PREEMPTED ID:", RETRIES_WHEN_PREEMPTED
    print "USE_AUTOSAVE ID:", USE_AUTOSAVE
    print "AUTOSAVE_FILENAME ID:", AUTOSAVE_FILENAME
    print "AUTOSAVE_CLEANUP ID:", AUTOSAVE_CLEANUP
    print "LOCATION_TAG ID:", LOCATION_TAG
    print "SHOW_TRACEBACKS ID:", SHOW_TRACEBACKS
    print "HOST_VERSION ID:", HOST_VERSION
    print "RENDERER_VERSION ID:", RENDERER_VERSION
    print "EMAILS ID:", EMAILS
    print "USE_EMAILS ID:", USE_EMAILS
    print "USE_FIXTURES ID:", USE_FIXTURES
    print "OVERRIDE_DESTINATION ID:", OVERRIDE_DESTINATION
    print "OVERRIDE_TASK_TEMPLATE ID:", OVERRIDE_TASK_TEMPLATE
    print "USE_SCRIPT:", USE_SCRIPT
    print "SCRIPT_FILENAME:", SCRIPT_FILENAME


class ConductorStore(object):
    """
    The store is used to persist a submission recipe in the scene file, and to
    repopulate the dialog when it's rebuilt.
    """

    def __init__(self):
        self.node = MaxPlus.INode.GetINodeByName(STORE_NAME)
        if not self.node:
            dummy = MaxPlus.Factory.CreateDummyObject()
            self.node = MaxPlus.Factory.CreateNode(dummy, STORE_NAME)
            self.reset()
        # print_ids()

    def clear(self):
        self.node.ClearAllAppData()

    def reset(self):

        self.clear()

        self.set_title(DEFAULT_TITLE)

        self.set_project(u"fishAttack")
        self.set_instance_type(u"n1-standard-16-t4-1")
        self.set_preemptible(True)
        self.set_chunk_size(1)
        self.set_use_custom_range(False)
        self.set_custom_range(u"1-10")
        self.set_use_scout_frames(True)
        self.set_scout_frames(u"auto:3")

        self.set_destination(_get_destination_dir())
        self.set_override_destination(False)

        self.set_task_template(DEFAULT_TASK_TEMPLATE)
        self.set_override_task_template(False)
        self.set_extra_environment()
        self.set_metadata()

        self.set_use_upload_daemon(False)
        self.set_upload_only(False)
        self.set_retries_when_preempted(1)

        self.set_use_autosave(True)
        self.set_autosave_filename(DEFAULT_AUTOSAVE_TEMPLATE)
        self.set_autosave_cleanup(True)

        self.set_location_tag("")
        self.set_emails(u"artist@example.com, producer@example.com")
        self.set_use_emails(False)

        self.set_show_tracebacks(False)
        self.set_use_fixtures(True)

        self.set_assets()
        self.set_host_version(u"maya-io 2018.SP0")
        self.set_renderer_version(u"ornatrix 2.2.5.17764")

        self.set_use_script(False)
        self.set_script_filename(DEFAULT_SCRIPT_COMMAND)

    def title(self):
        return self.node.GetAppData(TITLE)

    def set_title(self, value):
        try:
            self.node.DeleteAppData(TITLE)
        except RuntimeError:
            pass
        self.node.SetAppData(TITLE, value)

    def project(self):
        return self.node.GetAppData(PROJECT)

    def set_project(self, value):
        try:
            self.node.DeleteAppData(PROJECT)
        except RuntimeError:
            pass
        self.node.SetAppData(PROJECT, value)

    def instance_type(self):
        return self.node.GetAppData(INSTANCE_TYPE)

    def set_instance_type(self, value):
        try:
            self.node.DeleteAppData(INSTANCE_TYPE)
        except RuntimeError:
            pass
        self.node.SetAppData(INSTANCE_TYPE, value)

    def preemptible(self):
        return json.loads(self.node.GetAppData(PREEMPTIBLE))

    def set_preemptible(self, value):
        try:
            self.node.DeleteAppData(PREEMPTIBLE)
        except RuntimeError:
            pass
        self.node.SetAppData(PREEMPTIBLE, json.dumps(value))

    def override_destination(self):
        return json.loads(self.node.GetAppData(OVERRIDE_DESTINATION))

    def set_override_destination(self, value):
        try:
            self.node.DeleteAppData(OVERRIDE_DESTINATION)
        except RuntimeError:
            pass
        self.node.SetAppData(OVERRIDE_DESTINATION,  json.dumps(value))

    def destination(self):
        return self.node.GetAppData(DESTINATION)

    def set_destination(self, value):
        try:
            self.node.DeleteAppData(DESTINATION)
        except RuntimeError:
            pass
        self.node.SetAppData(DESTINATION, value)

    def chunk_size(self):
        return json.loads(self.node.GetAppData(CHUNK_SIZE))

    def set_chunk_size(self, value):
        try:
            self.node.DeleteAppData(CHUNK_SIZE)
        except RuntimeError:
            pass
        self.node.SetAppData(CHUNK_SIZE,  json.dumps(value))

    def use_custom_range(self):
        return json.loads(self.node.GetAppData(USE_CUSTOM_RANGE))

    def set_use_custom_range(self, value):
        try:
            self.node.DeleteAppData(USE_CUSTOM_RANGE)
        except RuntimeError:
            pass
        self.node.SetAppData(USE_CUSTOM_RANGE, json.dumps(value))

    def custom_range(self):
        return self.node.GetAppData(CUSTOM_RANGE)

    def set_custom_range(self, value):
        try:
            self.node.DeleteAppData(CUSTOM_RANGE)
        except RuntimeError:
            pass
        self.node.SetAppData(CUSTOM_RANGE, value)

    def use_scout_frames(self):
        return json.loads(self.node.GetAppData(USE_SCOUT_FRAMES))

    def set_use_scout_frames(self, value):
        try:
            self.node.DeleteAppData(USE_SCOUT_FRAMES)
        except RuntimeError:
            pass
        self.node.SetAppData(USE_SCOUT_FRAMES, json.dumps(value))

    def scout_frames(self):
        return self.node.GetAppData(SCOUT_FRAMES)

    def set_scout_frames(self, value):
        try:
            self.node.DeleteAppData(SCOUT_FRAMES)
        except RuntimeError:
            pass
        self.node.SetAppData(SCOUT_FRAMES, value)

    def override_task_template(self):
        return json.loads(self.node.GetAppData(OVERRIDE_TASK_TEMPLATE))

    def set_override_task_template(self, value):
        try:
            self.node.DeleteAppData(OVERRIDE_TASK_TEMPLATE)
        except RuntimeError:
            pass
        self.node.SetAppData(OVERRIDE_TASK_TEMPLATE,  json.dumps(value))

    def task_template(self):
        return self.node.GetAppData(TASK_TEMPLATE)

    def set_task_template(self, value):
        try:
            self.node.DeleteAppData(TASK_TEMPLATE)
        except RuntimeError:
            pass
        self.node.SetAppData(TASK_TEMPLATE, value)

    def extra_environment(self):
        return json.loads(self.node.GetAppData(EXTRA_ENVIRONMENT)) or []

    def set_extra_environment(self, obj=[]):
        try:
            self.node.DeleteAppData(EXTRA_ENVIRONMENT)
        except RuntimeError:
            pass
        self.node.SetAppData(EXTRA_ENVIRONMENT,  json.dumps(obj))

    def metadata(self):
        return json.loads(self.node.GetAppData(METADATA)) or []

    def set_metadata(self, obj=[]):
        try:
            self.node.DeleteAppData(METADATA)
        except RuntimeError:
            pass
        self.node.SetAppData(METADATA, json.dumps(obj))

    def use_upload_daemon(self):
        return json.loads(self.node.GetAppData(USE_UPLOAD_DAEMON))

    def set_use_upload_daemon(self, value):
        try:
            self.node.DeleteAppData(USE_UPLOAD_DAEMON)
        except RuntimeError:
            pass
        self.node.SetAppData(USE_UPLOAD_DAEMON, json.dumps(value))

    def upload_only(self):
        return json.loads(self.node.GetAppData(UPLOAD_ONLY))

    def set_upload_only(self, value):
        try:
            self.node.DeleteAppData(UPLOAD_ONLY)
        except RuntimeError:
            pass
        self.node.SetAppData(UPLOAD_ONLY,  json.dumps(value))

    def retries_when_preempted(self):
        return json.loads(self.node.GetAppData(RETRIES_WHEN_PREEMPTED))

    def set_retries_when_preempted(self, value):
        try:
            self.node.DeleteAppData(RETRIES_WHEN_PREEMPTED)
        except RuntimeError:
            pass
        self.node.SetAppData(RETRIES_WHEN_PREEMPTED, json.dumps(value))

    def use_autosave(self):
        return json.loads(self.node.GetAppData(USE_AUTOSAVE))

    def set_use_autosave(self, value):
        try:
            self.node.DeleteAppData(USE_AUTOSAVE)
        except RuntimeError:
            pass
        self.node.SetAppData(USE_AUTOSAVE, json.dumps(value))

    def autosave_filename(self):
        return self.node.GetAppData(AUTOSAVE_FILENAME)

    def set_autosave_filename(self, value):
        try:
            self.node.DeleteAppData(AUTOSAVE_FILENAME)
        except RuntimeError:
            pass
        self.node.SetAppData(AUTOSAVE_FILENAME, value)

    def autosave_cleanup(self):
        return json.loads(self.node.GetAppData(AUTOSAVE_CLEANUP))

    def set_autosave_cleanup(self, value):
        try:
            self.node.DeleteAppData(AUTOSAVE_CLEANUP)
        except RuntimeError:
            pass
        self.node.SetAppData(AUTOSAVE_CLEANUP, json.dumps(value))

    def location_tag(self):
        return self.node.GetAppData(LOCATION_TAG)

    def set_location_tag(self, value):
        try:
            self.node.DeleteAppData(LOCATION_TAG)
        except RuntimeError:
            pass
        self.node.SetAppData(LOCATION_TAG, value)

    def show_tracebacks(self):
        return json.loads(self.node.GetAppData(SHOW_TRACEBACKS))

    def set_show_tracebacks(self, value):
        try:
            self.node.DeleteAppData(SHOW_TRACEBACKS)
        except RuntimeError:
            pass
        self.node.SetAppData(SHOW_TRACEBACKS,  json.dumps(value))

    def use_fixtures(self):
        return json.loads(self.node.GetAppData(USE_FIXTURES))

    def set_use_fixtures(self, value):
        try:
            self.node.DeleteAppData(USE_FIXTURES)
        except RuntimeError:
            pass
        self.node.SetAppData(USE_FIXTURES, json.dumps(value))

    def host_version(self):
        return self.node.GetAppData(HOST_VERSION)

    def set_host_version(self, value):
        try:
            self.node.DeleteAppData(HOST_VERSION)
        except RuntimeError:
            pass
        self.node.SetAppData(HOST_VERSION, value)

    def renderer_version(self):
        return self.node.GetAppData(RENDERER_VERSION)

    def set_renderer_version(self, value):
        try:
            self.node.DeleteAppData(RENDERER_VERSION)
        except RuntimeError:
            pass
        self.node.SetAppData(RENDERER_VERSION, value)

    def use_emails(self):
        return json.loads(self.node.GetAppData(USE_EMAILS))

    def set_use_emails(self, value):
        try:
            self.node.DeleteAppData(USE_EMAILS)
        except RuntimeError:
            pass
        self.node.SetAppData(USE_EMAILS, json.dumps(value))

    def emails(self):
        return self.node.GetAppData(EMAILS)

    def set_emails(self, value):
        try:
            self.node.DeleteAppData(EMAILS)
        except RuntimeError:
            pass
        self.node.SetAppData(EMAILS, value)

    def assets(self):
        return json.loads(self.node.GetAppData(EXTRA_ASSETS))

    def set_assets(self, assets=[]):
        try:
            self.node.DeleteAppData(EXTRA_ASSETS)
        except RuntimeError:
            pass
        self.node.SetAppData(EXTRA_ASSETS, json.dumps(assets))

    def use_script(self):
        return json.loads(self.node.GetAppData(USE_SCRIPT))

    def set_use_script(self, value):
        try:
            self.node.DeleteAppData(USE_SCRIPT)
        except RuntimeError:
            pass
        self.node.SetAppData(USE_SCRIPT, json.dumps(value))

    def script_filename(self):
        return self.node.GetAppData(SCRIPT_FILENAME)

    def set_script_filename(self, value):
        try:
            self.node.DeleteAppData(SCRIPT_FILENAME)
        except RuntimeError:
            pass
        self.node.SetAppData(SCRIPT_FILENAME, value)


def _get_destination_dir():
    """Provide destination path - called by reset().
    """
    return Path(MaxPlus.PathManager.GetRenderOutputDir()).posix_path()



from PySide2 import QtWidgets, QtCore

from ciomax.components import widgets


class TextFieldGrp(QtWidgets.QWidget):

    def __init__(self, **kwargs):
        super(TextFieldGrp, self).__init__()
        self.hidable = kwargs.get("hidable")
        self.enablable = kwargs.get("enablable")
        self.is_directory = kwargs.get("directory")
        self.is_file = kwargs.get("file")
        self.custom_button = kwargs.get("custom_button", False)
        self.checkbox = None
        self.button = None
        

        if self.hidable:
            self.enablable = False


        placeholder = kwargs.get("placeholder")

        layout = QtWidgets.QHBoxLayout()
        self.setLayout(layout)

        layout.addWidget(
            widgets.FormLabel(
                kwargs.get("label", ""),
                tooltip=kwargs.get("tooltip")
                )
            )

        # hidable checkbox
        if self.hidable or self.enablable:
            self.display_checkbox = QtWidgets.QCheckBox()
            layout.addWidget(self.display_checkbox)
            self.display_checkbox.stateChanged.connect(self.set_active)


        # field
        self.field = QtWidgets.QLineEdit()
        self.field.setStyleSheet("QLineEdit { background: rgb(48, 48, 48); }")
        layout.addWidget(self.field)
        if placeholder:
            self.field.setPlaceholderText(placeholder)

        # optional checkbox or file browse button
        
        if self.is_directory or self.is_file:
            self.button = QtWidgets.QPushButton("Browse")
            self.button.setFixedWidth(80)
            layout.addWidget(self.button)
            self.button.clicked.connect(self.browse)
        elif self.custom_button:
            self.button = QtWidgets.QPushButton(self.custom_button)
            self.button.setFixedWidth(80)
            layout.addWidget(self.button)
        elif kwargs.get("checkbox"):
            self.checkbox =QtWidgets.QCheckBox(kwargs.get("check_label", ""))
            self.checkbox.setFixedWidth(80)
            layout.addWidget(self.checkbox)
        else:
            layout.addSpacing(85)



    def set_active(self, value=None):
        if not (self.hidable or self.enablable):
            return
        if self.hidable:
            self._show() if value else self._hide()

        self._enable() if value else self._disable()

    # TODO - Put all the extras in a widget and show/hide/enable/disable the widget
    def _hide(self):
        self.display_checkbox.setCheckState(QtCore.Qt.Unchecked)
        self.field.hide()
        if self.checkbox:
            self.checkbox.hide()
        if self.button:
            self.button.hide()

    def _show(self):
        self.display_checkbox.setCheckState(QtCore.Qt.Checked)
        self.field.show()
        if self.checkbox:
            self.checkbox.show()
        if self.button:
            self.button.show()

    def _disable(self):
        self.display_checkbox.setCheckState(QtCore.Qt.Unchecked)
        self.field.setEnabled(False)
        if self.checkbox:
            self.checkbox.setEnabled(False)
        if self.button:
            self.button.setEnabled(False)

    def _enable(self):
        self.display_checkbox.setCheckState(QtCore.Qt.Checked)
        self.field.setEnabled(True)
        if self.checkbox:
            self.checkbox.setEnabled(True)
        if self.button:
            self.button.setEnabled(True)

    def browse(self):
        if self.is_file:
            result = QtWidgets.QFileDialog.getOpenFileName()
        elif self.is_directory:
            result = QtWidgets.QFileDialog.getExistingDirectory()
        else:
            return
        if result:
            self.field.setText(result)


from PySide2 import QtWidgets, QtCore

from ciomax.components import widgets




class KeyValueHeaderGrp(QtWidgets.QWidget):
    """A header row"""
    def __init__(self, **kwargs):
        super(KeyValueHeaderGrp, self).__init__()

        layout = QtWidgets.QHBoxLayout()
        self.setLayout(layout)
        self.add_button = QtWidgets.QPushButton("Add")
        self.add_button.setFixedWidth(40)

        self.key_header = QtWidgets.QPushButton(kwargs.get("key_label", "Key"))
        policy = self.key_header.sizePolicy()
        policy.setHorizontalStretch(2)
        self.key_header.setSizePolicy(policy)
        self.key_header.setEnabled(False)

        self.value_header = QtWidgets.QPushButton(
            kwargs.get("value_label", "Value"))
        policy = self.value_header.sizePolicy()
        policy.setHorizontalStretch(3)
        self.value_header.setSizePolicy(policy)
        self.value_header.setEnabled(False)

        layout.addWidget(self.add_button)
        layout.addWidget(self.key_header)
        layout.addWidget(self.value_header)

        if kwargs.get("checkbox_label") is not None:
            self.excl_header = QtWidgets.QPushButton(
                kwargs.get("checkbox_label", "Active"))
            self.excl_header.setFixedWidth(40)
            self.excl_header.setEnabled(False)
            layout.addWidget(self.excl_header)
        else:
            layout.addSpacing(45)


class KeyValuePairGrp(QtWidgets.QWidget):
    """A single row"""
    delete_pressed = QtCore.Signal(QtWidgets.QWidget)

    def __init__(self, do_checkbox):
        super(KeyValuePairGrp, self).__init__()

        layout = QtWidgets.QHBoxLayout()
        self.willBeRemoved = False
        self.checkbox = None
        self.setLayout(layout)
        self.delete_button = QtWidgets.QPushButton("X")
        self.delete_button.setFixedWidth(40)

        self.delete_button.clicked.connect(self.delete_me)

        self.key_field = QtWidgets.QLineEdit()
        policy = self.key_field.sizePolicy()
        policy.setHorizontalStretch(2)
        self.key_field.setSizePolicy(policy)
 
        self.value_field = QtWidgets.QLineEdit()
        policy = self.value_field.sizePolicy()
        policy.setHorizontalStretch(3)
        self.value_field.setSizePolicy(policy)


        layout.addWidget(self.delete_button)
        layout.addWidget(self.key_field)
        layout.addWidget(self.value_field)

        if do_checkbox:
            self.checkbox = QtWidgets.QCheckBox()
            self.checkbox.setFixedWidth(40)
            layout.addWidget(self.checkbox)
        else:
            layout.addSpacing(45)

    def delete_me(self):
        self.delete_pressed.emit(self)


class KeyValueGrpList(QtWidgets.QWidget):
    """The list of KeyValuePairGrps"""

    edited = QtCore.Signal()


    def __init__(self, **kwargs):
        super(KeyValueGrpList, self).__init__()

        self.has_checkbox = kwargs.get("checkbox_label") is not None

        self.header_component = KeyValueHeaderGrp(**kwargs)
        self.content_layout = QtWidgets.QVBoxLayout()
        self.setLayout(self.content_layout)

        self.content_layout.addWidget(self.header_component)

        self.entries_component = QtWidgets.QWidget()
        self.entries_layout = QtWidgets.QVBoxLayout()
        self.entries_component.setLayout(self.entries_layout)
        self.content_layout.addWidget(self.entries_component)

        self.header_component.add_button.clicked.connect(self.add_entry)
        # self.header_component.add_button.clicked.connect(self.something_changed)

    def set_entries(self, entry_list):
        if self.has_checkbox:
            for row in  entry_list:
                self.add_entry(key=row[0], value=row[1], check=row[2])
        else:
            for row in  entry_list:
                self.add_entry(key=row[0], value=row[1])

    def add_entry(self, key="", value="", check=False):
        entry = KeyValuePairGrp(self.has_checkbox)
        entry.key_field.setText(key)
        entry.value_field.setText(value)
        if self.has_checkbox:
            entry.checkbox.setChecked(check)

        self.entries_layout.addWidget(entry)
        
        entry.delete_pressed.connect(remove_widget)
        entry.delete_pressed.connect(self.something_changed)
        entry.key_field.editingFinished.connect(self.something_changed)
        entry.value_field.editingFinished.connect(self.something_changed)
        if self.has_checkbox:
            entry.checkbox.stateChanged.connect(self.something_changed)

    def something_changed(self):
        self.edited.emit()


    def entry_widgets(self):
        return [e for e in self.entries_component.children() if e.metaObject().className() == "KeyValuePairGrp" and not e.willBeRemoved ]

    def entries(self):
        result = []
        for widget in self.entry_widgets():
            key = widget.key_field.text().strip()
            value = widget.value_field.text().strip()
            if key and value:
                if self.has_checkbox:
                    checked = widget.checkbox.isChecked()   
                    result.append([key, value, checked])
                else:
                    result.append([key, value])
        return result

    def clear(self):
        for entry in self.entries():
            remove_widget(entry)



@QtCore.Slot(QtWidgets.QWidget)
def remove_widget(widget):
    #  Since the widget is not deleted immediately, the call straight after this
    #  function which calls KeyValueGrpList.entries() and then updates the
    #  store, needs to know which widgets are left. If it simply gets children
    #  of the parent, it will include  this widget. By setting willBeRemoved, we
    #  can check it in KeyValueGrpList.entry_widgets() and then igfnore unwanted
    #  widgets when saving to the store.
    widget.willBeRemoved = True
    widget.layout().removeWidget(widget)
    widget.deleteLater()



from PySide2 import QtCore, QtWidgets
import re
import os
import shlex
from ciomax.sections.collapsible_section import CollapsibleSection
from ciomax.components.text_field_grp import TextFieldGrp
from ciomax.components.checkbox_grp import CheckboxGrp

from ciomax.components.int_field_grp import IntFieldGrp
from ciocore.expander import Expander
from ciocore.gpath import Path
import imp


class AdvancedSection(CollapsibleSection):
    ORDER = 100

    def __init__(self, dialog):
        super(AdvancedSection, self).__init__(dialog, "Advanced")

        self.autosave_component = TextFieldGrp(
            label="Autosave",
            checkbox=True,
            check_label="Clean up",
            hidable=True)
        self.content_layout.addWidget(self.autosave_component)

        self.task_template_component = TextFieldGrp(
            label="Task template",
            enablable=True)
        self.content_layout.addWidget(self.task_template_component)

        self.retry_preempted_component = IntFieldGrp(
            label="Preempted retries", default=1, minimum=0)
        self.content_layout.addWidget(self.retry_preempted_component)

        self.notification_component = TextFieldGrp(
            label="Send Notifications",
            enablable=True)
        self.content_layout.addWidget(self.notification_component)

        self.location_component = TextFieldGrp(
            label="Location tag")
        self.content_layout.addWidget(self.location_component)

        upload_options_component = CheckboxGrp(
            label="Uploads",
            checkboxes=2,
            sublabels=["Use upload daemon", "Upload only"]
        )
        self.use_daemon_checkbox = upload_options_component.checkboxes[0]
        self.upload_only_checkbox = upload_options_component.checkboxes[1]
        self.content_layout.addWidget(upload_options_component)

        separator = QtWidgets.QFrame()
        separator.setLineWidth(1)
        separator.setFrameStyle(QtWidgets.QFrame.HLine |
                                QtWidgets.QFrame.Raised)
        self.content_layout.addWidget(separator)

        diagnostics_options_grp = CheckboxGrp(
            label="Developer",
            checkboxes=2,
            sublabels=["Show tracebacks", "Use fixtures"]
        )

        self.tracebacks_checkbox = diagnostics_options_grp.checkboxes[0]
        self.fixtures_checkbox = diagnostics_options_grp.checkboxes[1]
        self.content_layout.addWidget(diagnostics_options_grp)

        self.script_component = TextFieldGrp(
            label="Pre submission script",
            enablable=True,
            custom_button="Execute",
            placeholder="Python script"
        )
        self.content_layout.addWidget(self.script_component)

        # self.autosave_component.checkbox.setEnabled(False)

        self.configure_signals()

    def configure_signals(self):
        """Write to store when values change"""

        # AUTOSAVE
        self.autosave_component.field.editingFinished.connect(
            self.on_autosave_change)

        self.autosave_component.checkbox.stateChanged.connect(
            self.on_autosave_cleanup_change)

        self.autosave_component.display_checkbox.stateChanged.connect(
            self.on_use_autosave_change)

        # TASK TEMPLATE
        self.task_template_component.field.editingFinished.connect(
            self.on_task_template_change)

        self.task_template_component.display_checkbox.stateChanged.connect(
            self.on_override_task_template_change)

        self.retry_preempted_component.field.valueChanged.connect(
            self.on_retry_preempted_change)

        self.notification_component.field.editingFinished.connect(
            self.on_notification_change)

        self.notification_component.display_checkbox.stateChanged.connect(
            self.on_use_notification_change)

        self.location_component.field.editingFinished.connect(
            self.on_location_change)

        # DAEMON
        self.use_daemon_checkbox.clicked.connect(self.on_use_daemon_change)

        self.upload_only_checkbox.clicked.connect(self.on_upload_only_change)

        self.tracebacks_checkbox.clicked.connect(
            self.on_show_tracebacks_change)

        self.fixtures_checkbox.clicked.connect(
            self.on_use_fixtures_change)

        self.script_component.display_checkbox.stateChanged.connect(
            self.on_use_script_change)

        self.script_component.field.editingFinished.connect(
            self.on_script_change)

        self.script_component.button.clicked.connect(
            self.on_script_button)

    def on_autosave_change(self):
        self.dialog.store.set_autosave_filename(
            self.autosave_component.field.text())

    def on_autosave_cleanup_change(self, value):
        self.dialog.store.set_autosave_cleanup(value > 0)

    def on_use_autosave_change(self, value):
        self.dialog.store.set_use_autosave(value > 0)

    def on_task_template_change(self):
        self.dialog.store.set_task_template(
            self.task_template_component.field.text())

    def on_override_task_template_change(self, value):
        self.dialog.store.set_override_task_template(value > 0)

    def on_retry_preempted_change(self, value):
        self.dialog.store.set_retries_when_preempted(value)

    def on_notification_change(self):
        self.dialog.store.set_emails(self.notification_component.field.text())

    def on_use_notification_change(self, value):
        self.dialog.store.set_use_emails(value > 0)

    def on_location_change(self):
        self.dialog.store.set_location_tag(
            self.location_component.field.text())

    def on_use_daemon_change(self, value):
        self.dialog.store.set_use_upload_daemon(value > 0)
        self.autosave_component.checkbox.setEnabled(not value)

    def on_upload_only_change(self, value):
        self.dialog.store.set_upload_only(value > 0)

    def on_show_tracebacks_change(self, value):
        self.dialog.store.set_show_tracebacks(value > 0)

    def on_use_fixtures_change(self, value):
        self.dialog.store.set_use_fixtures(value > 0)

    def on_use_script_change(self, value):
        self.dialog.store.set_use_script(value > 0)

    def on_script_change(self):
        self.dialog.store.set_script_filename(
            self.script_component.field.text())

    def on_script_button(self):
        result = self.run_presubmit_script()
        print result

    def run_presubmit_script(self):

        context = self.dialog.main_tab.get_context()
        main_sequence = self.dialog.main_tab.section(
            "FramesSection").main_sequence
        context.update({
            "start": main_sequence.start,
            "end": main_sequence.end
        })
        expander = Expander(safe=True, **context)

        print "context is", context
        cmd = expander.evaluate(unicode(self.script_component.field.text()))
        print "Command is:", cmd

        parts = shlex.split(cmd)
        if not parts:
            raise ValueError("Enter full path to script followed by args")

        script_path = Path(parts[0]).posix_path()
        print "script_path is:", script_path
        args = parts[1:]
        try:
            script_module = imp.load_source("", script_path)
        except ImportError:
            raise ImportError(
                "Can't load the pre-submit script '{}' as a Python module.".format(script_path))
        except SyntaxError:
            raise SyntaxError(
                "Syntax error in the pre-submit script: '{}'.".format(script_path))
        return script_module.main(*args)

    def populate_from_store(self):
        store = self.dialog.store
        self.autosave_component.field.setText(store.autosave_filename())

        self.autosave_component.set_active(store.use_autosave())
        self.autosave_component.checkbox.setChecked(store.autosave_cleanup())

        self.task_template_component.field.setText(store.task_template())
        self.task_template_component.set_active(store.override_task_template())

        self.retry_preempted_component.field.setValue(
            store.retries_when_preempted())

        self.notification_component.field.setText(store.emails())
        self.notification_component.set_active(store.use_emails())

        self.location_component.field.setText(store.location_tag())

        self.upload_only_checkbox.setChecked(store.upload_only())

        # USE DAEMON: Autosave cleanup must be disabled if use daemon
        use_daemon = store.use_upload_daemon()
        self.use_daemon_checkbox.setChecked(use_daemon)
        self.autosave_component.checkbox.setEnabled(not use_daemon)

        self.tracebacks_checkbox.setChecked(store.show_tracebacks())
        self.fixtures_checkbox.setChecked(store.use_fixtures())

        self.script_component.set_active(store.use_script())
        self.script_component.field.setText(store.script_filename())

        return

    def resolve(self, expander):
        emails = re.split(r"[, ]+",  self.notification_component.field.text())
        emails = [email for email in emails if email]
        location = expander.evaluate(self.location_component.field.text())

        result = {}

        result["autoretry_policy"] = {
            "preempted": {
                "max_retries": self.retry_preempted_component.field.value()
            }
        }

        if emails:
            result["notify"] = emails

        if location:
            result["location"] = location

        result["local_upload"] = not self.use_daemon_checkbox.isChecked(),
        result["upload_only"] = self.upload_only_checkbox.isChecked()

        return result



from PySide2 import QtWidgets, QtGui, QtCore

from ciomax.sections.collapsible_section import CollapsibleSection
from ciomax.components.key_value_grp import KeyValueGrpList


class MetadataSection(CollapsibleSection):
    ORDER = 70

    def __init__(self, dialog):
        super(MetadataSection, self).__init__(dialog, "Metadata")

        self.component = KeyValueGrpList()
        self.content_layout.addWidget(self.component)
        self.configure_signals()

    def configure_signals(self):
        """Write to store when values change"""
        self.component.edited.connect(self.on_edited)

    def on_edited(self):
        self.dialog.store.set_metadata(self.component.entries())

    def populate_from_store(self):
        store = self.dialog.store
        self.component.set_entries(store.metadata())

    def resolve(self, expander, **kwargs):
        result = {}
        for entry in self.component.entries():
            result[entry[0]] = expander.evaluate(entry[1])
        return {"metadata": result}
    



# from __future__ import unicode_literals


import os
import sys
import errno

# /users/me/Conductor/ciomax
PKG_DIR = os.path.dirname(os.path.abspath(__file__))
PKGNAME = os.path.basename(PKG_DIR)
PLATFORM = sys.platform
# LOCALAPPDATA = os.environ.get(  "LOCALAPPDATA", "%LOCALAPPDATA%")

LOCALAPPDATA =  os.path.expanduser(os.path.join("~","AppData", "Local"))

STUB_SCRIPT_FILENAME = "ciomax_stub.ms"

MENU_SCRIPT_PATH = os.path.join(PKG_DIR, "Conductor.ms").replace('\\', r'\\')


def main():

    content = """
-- Calls the Conductor 3DS Max plugin. Do not modify.
(
  include "{}"
)
""".format(MENU_SCRIPT_PATH)

    if not PLATFORM in ["win32"]:
        sys.stderr.write("Unsupported platform: {}\n".format(PLATFORM))
        manual_instructions(content)
        sys.exit(0)

    path = os.path.join(LOCALAPPDATA, "Autodesk", "3dsMax")
    if not os.path.exists(path):
        sys.stderr.write(
            "No 3ds startup directory in your local app data: {}\n".format(PLATFORM))
        manual_instructions(content)
        sys.exit(0)

    for folder in [f for f in os.listdir(path) if f.startswith("20")]:
        path = os.path.join(LOCALAPPDATA, "Autodesk", "3dsMax",
                            folder, "ENU", "scripts", "startup")
        if os.path.exists(path):
            ensure_directory(path)

        stub_file = os.path.join(path, STUB_SCRIPT_FILENAME)
        with open(stub_file, "w") as fn:
            fn.write(content)


def manual_instructions(content):
    msg = """MANUAL INSTRUCTIONS:
    If you are installing on a platform other than Windows, 
    or you haven't yet installed 3ds Max, then you'll need 
    to create a startup file manually so that 3ds Max can 
    find the Conductor menu script.

    The contents of the startup file should be something like:
    {}
    (You may need to adjust for Windows directory mapping.)

    The file should be placed here:
    {}
    """.format(content, os.path.join("%LOCALAPPDATA%", "Autodesk", "3dsMax", "<3ds-max-version>", "ENU", "scripts", "startup",STUB_SCRIPT_FILENAME))

    sys.stdout.write(msg)


def ensure_directory(directory):
    try:
        os.makedirs(directory)
    except OSError as ex:
        if ex.errno == errno.EEXIST and os.path.isdir(directory):
            pass
        else:
            raise


if __name__ == '__main__':
    main()



from PySide2 import QtWidgets, QtGui

from ciomax.sections.collapsible_section import CollapsibleSection
from ciomax.components.software_combo_box_grp import SoftwareComboBoxGrp
from ciocore import data as coredata
from ciocore.package_environment import PackageEnvironment


class SoftwareSection(CollapsibleSection):
    ORDER = 20

    def __init__(self, dialog):

        super(SoftwareSection, self).__init__(
            dialog, "Software", expanded=True)

        self.component = SoftwareComboBoxGrp()
        self.content_layout.addWidget(self.component)
        self.configure_combo_boxes()

        # Write to store when values change
        self.component.combobox_renderer_version.currentTextChanged.connect(
            self.on_change)

    def on_change(self, value):

        store = self.dialog.store
        store.set_host_version(self.component.combobox_host.currentText())
        store.set_renderer_version(
            "{} {}".format(
                self.component.combobox_renderer_name.currentText(),
                self.component.combobox_renderer_version.currentText()
            )
        )

    def populate_from_store(self):
        store = self.dialog.store

        # We have to save the renderer from the store in a variable before
        # setting the host, because when we set the host, a cascade of events
        # take place which ultimately set the store value.
        # TODO : Use activated() instead of currentTextChanged()
        host_version = store.host_version()
        renderer_version = store.renderer_version()

        self.component.set_host_by_text(host_version)
        self.component.set_renderer_by_text(renderer_version)

    def configure_combo_boxes(self):
        if not coredata.valid():
            print "Invalid packages data"
            return False

        model = QtGui.QStandardItemModel()
        software_data = coredata.data()["software"]
        hosts = software_data.supported_host_names()
        for host in hosts:
            host_item = QtGui.QStandardItem(host)
            plugins = software_data.supported_plugins(host)
            for plugin in plugins:
                plugin_item = QtGui.QStandardItem(plugin["plugin"])
                for version in plugin["versions"]:
                    version_item = QtGui.QStandardItem(version)
                    #
                    plugin_item.appendRow(version_item)
                host_item.appendRow(plugin_item)
            model.appendRow(host_item)

        self.component.set_model(model)

    def resolve(self, expander, **kwargs):
        extra_environment = self.dialog.main_tab.section(
            "EnvironmentSection").get_entries(expander)

        packages_data = self.get_packages()
        if not packages_data:
            return []

        packages_data["env"].extend(extra_environment)
        return {
            "environment": dict(packages_data["env"]),
            "software_package_ids": packages_data["ids"]
        }

    def get_packages(self):
        if not coredata.valid():
            return
        tree_data = coredata.data()["software"]
        paths = []
        host_path = self.component.combobox_host.currentText()

        renderer_path = "{}/{}".format(host_path,
                                       self.component.get_renderer())
        paths = [host_path, renderer_path]

        result = {
            "ids": [],
            "env": PackageEnvironment()
        }
        for package in filter(None, [tree_data.find_by_path(path) for path in paths if path]):
            result["ids"].append(package["package_id"])
            result["env"].extend(package)

        return result



import MaxPlus



from PySide2 import QtWidgets
from ciomax.sections.collapsible_section import CollapsibleSection
from ciocore.gpath_list import GLOBBABLE_REGEX, PathList

class ExtraAssetsSection(CollapsibleSection):
    ORDER = 65

    def __init__(self,dialog):
        super(ExtraAssetsSection, self).__init__(dialog,"Extra Assets")

        # Buttons
        self.button_layout = QtWidgets.QHBoxLayout()

        for button in [
            {"label": "Clear", "func": self.clear},
            {"label": "Remove selected", "func": self.remove_selected},
            {"label": "Browse files", "func": self.browse_files},
            {"label": "Browse directory", "func": self.browse_dir},
        ]:

            btn = QtWidgets.QPushButton(button["label"])
            btn.clicked.connect(button["func"])
            self.button_layout.addWidget(btn)

        self.content_layout.addLayout(self.button_layout)

        # List
        self.list_component = QtWidgets.QListWidget()
        self.list_component.setSelectionMode(
            QtWidgets.QAbstractItemView.ExtendedSelection)
        self.list_component.setFixedHeight(140)
        self.content_layout.addWidget(self.list_component)
 
    def add_paths(self, *paths):
        # use a PathList to deduplicate.
        path_list = PathList(*self.entries())
        path_list.add(*paths)
        self.list_component.clear()
        self.list_component.addItems([p.posix_path() for p in path_list])


    def entries(self):
        result = []
        for i in range(self.list_component.count() ):
            result.append(self.list_component.item(i).text())
        return result

    def clear(self):
        self.list_component.clear()
        self.dialog.store.set_assets([])

    def remove_selected(self):
        model = self.list_component.model()
        for row in sorted([index.row() for index in self.list_component.selectionModel().selectedIndexes()], reverse=True):
            model.removeRow(row)

        self.dialog.store.set_assets( self.entries() )

    def browse_files(self):
        result = QtWidgets.QFileDialog.getOpenFileNames(
            parent=None, caption="Select files to upload")
        if len(result) and len(result[0]):
            self.add_paths(*result[0])
            # self.list_component.addItems(result[0])
            self.dialog.store.set_assets( self.entries() )

    def browse_dir(self):
        result = QtWidgets.QFileDialog.getExistingDirectory(
            parent=None, caption="Select a directory to upload")
        if result:
            self.add_paths(result)
            # self.list_component.addItem(result)
            self.dialog.store.set_assets( self.entries() )

    def populate_from_store(self):
        self.list_component.addItems(self.dialog.store.assets())


 
    def resolve(self, expander, **kwargs):
 
   
        path_list = PathList()
        path_list.add(*self.entries())
        max_assets = [asset.GetResolvedFileName() for asset in list(MaxPlus.AssetManager.GetAssets())]

        path_list.add(*max_assets)

        scenefile =  expander.evaluate("<scenefile>")
        
        path_list.add(scenefile)


        # in place expand glob characters
        path_list.glob()

        return  {"upload_paths":  [p.posix_path() for p in path_list]}
 




def main(*args):
    print "-" *30
    print args
    print "-" *30
    return {
        "extra_assets": [
            "/path1",
            "/path2"
        ]
    }


########## KEEP FOR NOW WHILE WE FIGURE OUT HOW TO EXPORT ASS
#  commands = [
#     # Render Setup window must be closed, otherwise setting rendTimeType and
#     # rendPickupFrames might not stick, according to Autodesk
#     # https://help.autodesk.com/view/3DSMAX/2017/ENU/?guid=__files_GUID_30AF1E53_5A69_402D_84D6_4D6ECCDD6D20_htm
#     'renderSceneDialog.close()',
#     # rendTimeType 4 is equivalent to "Frames" in
#     # "Render Setup -> Common -> TimeOutput" and it expects the frame range
#     # to be specified via rendPickupFrames
#     'rendTimeType=4',
#     'rendPickupFrames="%s-%s"' % (start_frame, end_frame),
#     'rendNThFrame=1',
#     'renderers.current.export_to_ass = true',
#     'renderers.current.ass_file_path = "%s"' % file_name,
#     'render fromFrame:%s toFrame:%s vfb:false' % (start_frame, end_frame)
# ]

# command = ';'.join(commands)
# maybe_error = MaxPlus.FPValue()
# if not MaxPlus.Core.EvalMAXScript(command, maybe_error):
#     raise RuntimeError(str(maybe_error))

# import MaxPlus

# # MaxPlus.Core.EvalMAXScript(command, maybe_error):
    
    
# import pymxs
# rt = pymxs.runtime

# print "initialized"
# print rt.renderers
# print dir(rt.renderers)
# print rt.renderers.current
# print dir(rt.renderers.current)
# print rt.renderers.current.ass_file_path
# rt.renderers.current.ass_file_path = "W:/perishable/max_proj/foo"
# print rt.renderers.current.export_to_ass
# rt.renderSceneDialog.close()

# rt.renderers.current = rt.Arnold

# rt.renderers.current = rt.RendererClass.classes[0]()
# print rt.renderers.current

# print dir(rt.RendererClass.classes[0]())

# print rt.RendererClass.classes
# for klass in rt.RendererClass.classes:
#     print dir(klass)
    
    
# print rt.Arnold